/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Fri Jan 17 17:59:14 UTC 2003
    copyright            : (C) 2003 by Oleksiy Kohany
    email                : kohany@girinst.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

//#define DEBUG

#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <stdlib.h>
#include <string.h>
#include "constants.h"
#include "functions.h"
#include "cdb.h"
#include <stdexcept>
#include <map>
#include <set>
using namespace std;
using namespace bio;

typedef set<string> STRSET;

/** ======================================================== */
void usage(char* exec) {
   if(!exec) {
      return;
   }
   if(strrchr(exec, '/')) {
      exec = strrchr(exec, '/') + 1;
   }
   cerr << "NAME\n\t" << exec;
#ifdef VERSION
   cerr << " V" << VERSION;
#endif
   cerr << " - extracts sequences by locus name\n";
   cerr << "\nSYNOPSIS\n";
   cerr << "\t" << exec << " [locuslist] db_or_index_file [outfile]\n";
   cerr << "\nDESCRIPTION\n";
   cerr << "\tExtracts sequences from multisequence database file by\n";
   cerr << "\ttheir locus names. By default extracts each sequence only once.\n";
   cerr << "\nFLAGS\n";
   cerr << "\t-asis - this option causes each sequence to be extracted\n";
   cerr << "\tas many times as it's name appears on the list.\n";
#ifdef PACKAGE_BUGREPORT
   cerr << "\nBUG REPORT\n\t" << PACKAGE_BUGREPORT << "\n";
#endif
#ifdef COPYRIGHT
   cerr << "\nCOPYRIGHT\n\t" << COPYRIGHT << "\n";
#endif
}
/** ======================================================== */
int main(int argc, char *argv[])
{
   istream *is;
   ostream *os;
   bool getUnique = true;
   ARGMAP amap;
   char sDBName[MAXNAMESIZE];
   ::getflags(argv, amap, 3, "--help", "--version", "-asis");
   if(amap.find("--help") != amap.end() || amap.find("--version") != amap.end()) {
      usage(argv[0]);
      exit(0);
   }
   if(amap.find("-asis") != amap.end()) {
     getUnique = false;
   }
   char **args = argv;
   if((args = assigninput(is, args)) == 0) {
      usage(argv[0]);
      exit(10);
   }
   if(args[1] == 0) {
      usage(argv[0]);
      exit(20);
   }
   strcpy(sDBName, args[1]);
   args++;
   if((args = assignoutput(os, args)) == 0) {
      usage(argv[0]);
      exit(40);
   }
//========================================================
   int iRes = 1;
   try {
      CDb ddb(sDBName);
      STRSET done;
      string sLine;
      string sDBL;
      istream *ids;
      istringstream strstream;
      while(std::getline(*is, sLine) && !is->fail()) {
         strstream.str(sLine);
         strstream.seekg(0, ios::beg);
         strstream.clear();
         strstream >> sDBL;
         if(sDBL.size() == 0) {
            cerr << "Bad list row: \n" << sLine;
            continue;
         }
         if(sDBL[sDBL.size() - 1] == ':') {
           sDBL.erase(sDBL.size() - 1);
         }
         if(getUnique && done.find(sDBL) != done.end()) {
            //cerr << "Duplicated loci " << sDBL << "\n";
            continue;
         }
         try {
            istream& ids(ddb.getHead(sDBL.c_str()));
            string sName;
            if(getline(ids, sLine) && !ids.fail()) {
               *os << sLine << "\n";
               switch(sLine[0]) {
                  case '>':
                  strstream.str(sLine);
                  strstream.seekg(0, ios::beg);
                  strstream.clear();
                  strstream >> sName[0];
                  strstream >> sName;
                  break;

                  case ';':
                  while(getline(ids, sLine) && !ids.fail() && sLine[0] == ';') {
                     *os << sLine << "\n";
                     sLine[0] = '\0';
                  }
                  *os << sLine << "\n";
                  strstream.str(sLine);
                  strstream.seekg(0, ios::beg);
                  strstream.clear();
                  strstream >> sName;
                  break;

                  default:
                     cerr << sLine << "\n";
                     cerr << "FATAL: Bad data file or outdated index.\n";
                     exit(50);
                  return 0;
               }
               if(sDBL != sName) {
                  cerr << "FATAL: Found " << sName << " instead of " << sDBL << "\n";
                  exit(60);
               }
               sLine[0] = '\0';
               while(ids.peek() != ';' && ids.peek() != '>' && getline(ids, sLine) && !ids.fail()) {
                  *os << sLine << "\n";
               }
               os->flush();
               done.insert(sName);
               iRes *= 0;
            }
//            ids->clear();
         }
         catch(out_of_range err) {
            cerr << err.what() << endl;
         }
      }
   }
   catch(invalid_argument& exc) {
      cerr << argv[0] << " : main() : badfilename '" << exc.what() << "'\n";
      exit(50);
   }
//========================================================
   if(is != &cin) {
      delete is;
   }
   if(os != &cout) {
      delete os;
   }
   exit(iRes);
}
