/***************************************************************************
 main.cpp  -  description
 -------------------
 begin                : Mon Dec 16 11:42:34 PST 2002
 copyright            : (C) 2002 by Oleksiy Kohany
 email                : kohany@girinst.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <stdlib.h>
#include <string.h>
#include "functions.h"
#include "BioSequence.h"
#include "constants.h"
using namespace std;
using namespace bio;
class rc;

/**
 ======================================================
 Class rc - one record into conversion map.
 ======================================================
 */
class rc {
   public:
      rc() {
         SRC = DST = "";
         iSF = iST = iDF = iDT = 0;
      }
      rc(const string src, unsigned long isf, unsigned long ist, const string dst, unsigned long idf, unsigned long idt) {
         assign(src, isf, ist, dst, idf, idt);
      }
      void assign(const string src, unsigned long isf, unsigned long ist, const string dst, unsigned long idf, unsigned long idt) {
         SRC = src;
         DST = dst;
         iSF = isf;
         iST = ist;
         iDF = idf;
         iDT = idt;
      }
      ;
      /** No descriptions */
      void print(ostream& os) const {
         if (iSF + iST + iDF + iDT > 0) {
            os << DST << "\t" << iDF << "\t" << iDT << "\t" << SRC << "\t" << iSF << "\t" << iST << "\n";
         }
      }
      /** No descriptions */
      void print(FILE* os) const {
         if (iSF + iST + iDF + iDT > 0) {
            fprintf(os, "%s\t%ld\t%ld\t%s\t%ld\t%ld\n", DST.c_str(), iDF, iDT, SRC.c_str(), iSF, iST);
         }
      }

      string SRC;
      string DST;
      unsigned long iSF;
      unsigned long iST;
      unsigned long iDF;
      unsigned long iDT;
};

void ssplit(istream& is, ostream& os, ostream& smap, unsigned long iMXSZ, unsigned long iNSZ) {
   BioSequence seq1;
   BioSequence seq2;
   string sLine;
   string sName;
   istringstream iss;
   ostringstream oss;
   rc row;
   unsigned int fcnt = 0;
   while (!getline(is, sLine).fail()) {
      if (sLine[0] == '>' || sLine[0] == ';') {
         if (seq2.size() > 0) {
            if (fcnt > 0) {
               oss.clear();
               oss.str("");
               oss << sName << "_" << ++fcnt;
               seq2.setName(oss.str());
               seq2.print(os);
               row.DST = oss.str();
               row.iDT = seq2.size();
               row.iST = row.iSF + seq2.size() - 1;
               row.print(smap);
            } else {
               seq2.print(os);
            }
            seq2.clear();
         }
         if (sLine[0] == '>') {
            sLine.erase(0, 1);
         } else {
            while (!getline(is, sLine).eof() && sLine[0] == ';')
               ;
         }
         iss.clear();
         iss.str(sLine);
         iss >> sName;
         fcnt = 0;
         row.assign(sName, 1, 0, sName, 1, 0);
         seq2.setName(sName);
         continue;
      }
      if (*(sLine.end() - 1) == '\n') {
         sLine.erase(sLine.end() - 1);
      }
      if (*(sLine.end() - 1) == '1') {
         sLine.erase(sLine.end() - 1);
      }
      seq2.append(sLine);
      if (seq2.size() > iMXSZ) {
         seq1 = seq2.substr(0, iNSZ);
         oss.clear();
         oss.str("");
         oss << sName << "_" << ++fcnt;
         seq1.setName(oss.str());
         seq1.print(os);
         row.DST = oss.str();
         row.iST = row.iSF + seq1.size() - 1;
         row.iDT = seq1.size();
         row.print(smap);
         seq1.clear();
         seq2.erase(0, iNSZ);
         row.iSF = row.iST + 1;
         row.iDF = 1;
         row.iDT = 0;
      }
   }
   if (fcnt > 0) {
      oss.clear();
      oss.str("");
      oss << sName << "_" << ++fcnt;
      seq2.setName(oss.str());
      seq2.print(os);
      row.DST = oss.str();
      row.iST = row.iSF + seq2.size() - 1;
      row.iDT = seq2.size();
      row.print(smap);
   } else {
      seq2.print(os);
   }
}

void usage(char* exec) {
   if (!exec) {
      return;
   }
   if (strrchr(exec, '/')) {
      exec = strrchr(exec, '/') + 1;
   }
   cerr << "NAME\n\t" << exec;
#ifdef VERSION
   cerr << " V" << VERSION;
#endif
   cerr << " - splitts sequences\n";
   cerr << "\nSYNOPSIS\n\t" << exec << " [infile] mmap normsize maxsize [outfile]\n\n";
   cerr << "DESCRIPTION\n\tSplitts genomic sequences into peaces of size normsize.\n\n";
   cerr << "\tmmap - is a backward conversion map.\n";
#ifdef COPYRIGHT
   cerr << "\nCOPYRIGHT\n\t" << COPYRIGHT << "\n";
#endif
}

int main(int argc, char *argv[]) {
   istream *is;
   ostream *os;
   ofstream *ofs;
   char** args = argv;
   unsigned long iMXSZ, iNSZ;
   if ((args = assigninput(is, args)) == 0) {
      usage(argv[0]);
      exit(10);
   }
   if ((ofs = openofs(args[1])) == 0) {
      usage(argv[0]);
      exit(20);
   }
   args++;
   if (args[1] == 0 || sscanf(args[1], "%ld", &iNSZ) < 1) {
      usage(argv[0]);
      exit(30);
   }
   args++;
   if (args[1] == 0 || sscanf(args[1], "%ld", &iMXSZ) < 1) {
      usage(argv[0]);
      exit(40);
   }
   args++;
   if ((args = assignoutput(os, args)) == 0) {
      usage(argv[0]);
      exit(50);
   }
   if (iMXSZ < iNSZ) {
      cerr << "'maxsize' has to be more than 'normsize'.\n";
      exit(60);
   }
   //========================================================================
   ssplit(*is, *os, *ofs, iMXSZ, iNSZ);
   //========================================================================
   os->flush();
   ofs->close();
   return 0;
}
